<a name="table_engines-dictionary"></a>

# Dictionary

Движок `Dictionary` отображает данные словаря как таблицу ClickHouse.

Рассмотрим для примера словарь `products` со следующей конфигурацией:

```xml
<dictionaries>
<dictionary>
        <name>products</name>
        <source>
            <odbc>
                <table>products</table>
                <connection_string>DSN=some-db-server</connection_string>
            </odbc>
        </source>
        <lifetime>
            <min>300</min>
            <max>360</max>
        </lifetime>
        <layout>
            <flat/>
        </layout>
        <structure>
            <id>
                <name>product_id</name>
            </id>
            <attribute>
                <name>title</name>
                <type>String</type>
                <null_value></null_value>
            </attribute>
        </structure>
</dictionary>
</dictionaries>
```

Запрос данных словаря:

```sql
select name, type, key, attribute.names, attribute.types, bytes_allocated, element_count,source from system.dictionaries where name = 'products';                     

SELECT 
    name, 
    type, 
    key, 
    attribute.names, 
    attribute.types, 
    bytes_allocated, 
    element_count, 
    source
FROM system.dictionaries 
WHERE name = 'products'
```
```
┌─name─────┬─type─┬─key────┬─attribute.names─┬─attribute.types─┬─bytes_allocated─┬─element_count─┬─source──────────┐
│ products │ Flat │ UInt64 │ ['title']       │ ['String']      │        23065376 │        175032 │ ODBC: .products │
└──────────┴──────┴────────┴─────────────────┴─────────────────┴─────────────────┴───────────────┴─────────────────┘
```

В таком виде данные из словаря можно получить при помощи функций [dictGet*](../functions/ext_dict_functions.md#ext_dict_functions).

Такое представление неудобно, когда нам необходимо получить данные в чистом виде, а также при выполнении операции `JOIN`. Для этих случаев можно использовать движок `Dictionary`, который отобразит данные словаря в таблицу.

Синтаксис:

```
CREATE TABLE %table_name% (%fields%) engine = Dictionary(%dictionary_name%)`
```


Пример использования:

```sql
create table products (product_id UInt64, title String) Engine = Dictionary(products);

CREATE TABLE products
(
    product_id UInt64, 
    title String, 
)
ENGINE = Dictionary(products)
```
```
Ok.

0 rows in set. Elapsed: 0.004 sec. 
```

Проверим что у нас в таблице?

```sql
select * from products limit 1;

SELECT *
FROM products 
LIMIT 1
```
```
┌────product_id─┬─title───────────┐
│        152689 │ Некоторый товар │
└───────────────┴─────────────────┘

1 rows in set. Elapsed: 0.006 sec. 
```
